from WkLog import log


def before_execute(func):
    def wrapper(*args, **kwargs):
        print(f"before_execute -> {func.__name__}")
        print(f"args: {args}")
        print(f"kwargs: {kwargs}")
        # self.__test_conn()
        if not args and not kwargs:
            log.error(f"{func.__name__} -> 参数不能为空！")
            return False

        if args and kwargs:
            log.error(f"{func.__name__} -> 参数不能同时存在！")

        if args:
            if len(args) > 1:
                log.error(f"{func.__name__} -> 参数个数不能超过1")
            if not isinstance(args[0], dict):
                log.error(f"{func.__name__} -> 参数必须是字典！")
        result = func(*args, **kwargs)
        return result

    return wrapper


@before_execute
def func(x, y, *args, **kwargs):
    print(f"x={x}")
    print(f"y={y}")
    print(f"args={args}")
    print(f"kwargs={kwargs}")
    pass


# def exists(x, y):
#     print(f"exists: {x} - {y}")


# def exists(x: dict):
#     print(f"exists: {x}")


@before_execute
def exists(*args, **kwargs):
    print(f"exists-args: {args} - {True if args else False}")
    print(f"exists-kwargs: {kwargs} - {True if kwargs else False}")


if __name__ == "__main__":
    pass
    # func(x=1, y=2, a=1, b=2, c=3)
    # func(1, 2, a=1, b=2, c=3)
    # func(1, 2, 3, 4, 5, a=1, b=2, c=3)
    # kwargs = {"a": 1, "b": 2, "c": 3}
    # func(1, 2, 3, 4, 5, **kwargs)
    # print(*kwargs)
    # exists(1, 2)
    # exists(1, 2, 3, 4, 5)
    # exists(1)
    # exists({"a": 1, "b": 2})
    # exists(a=1, b=2)
    # exists(id=1, name="wangkang")
    # exists({"id": 1, "name": "wangkang"})



from WkMysql import *
from WkMysql.WkMysqlPool import WkMysqlPool

# 多线程
import threading
import time
import random


def test_multi_thread_select():
    db = DB()
    db.set_table(TABLE)
    start = time.time()

    tasks = []
    for i in range(1000):
        tasks.append(threading.Thread(target=db.select, args=({"id": str(random.randint(31, 100))},)))

    for task in tasks:
        task.start()

    for task in tasks:
        task.join()

    end = time.time()
    print("用时：", end - start)


def test_multi_thread_select2():
    def select(name):
        print(f"thread {name} start")
        conn = pool.get_conn()
        print(f"thread {name} get conn")
        res = conn.set_table(TABLE).select(sno="444444444444444444")
        # time.sleep(9.5)
        pool.close_all()
        pool.put_conn(conn)
        print(f"thread {name} - {res} - {pool.pool}")

    pool = WkMysqlPool(
        host=HOST,
        user=USER,
        port=PORT,
        password=PASSWORD,
        database=DATABASE,
        max_conn=5,
        min_conn=1,
    )
    start = time.time()

    tasks = []
    for i in range(100):
        tasks.append(threading.Thread(target=select, args=(str(i),)))

    for task in tasks:
        task.start()

    for task in tasks:
        task.join()

    end = time.time()
    print("用时：", end - start)


if __name__ == "__main__":
    pass
    # test_multi_thread_select()
    test_multi_thread_select2()



from WkMysql import WkMysql
from pymysql.cursors import Cursor
import time

HOST = "localhost"
PORT = 3306
USER = "root"
PASSWORD = "123456"
DATABASE = "myproject"
TABLE = "test_table"

if __name__ == "__main__":
    # db = DB(cursorclass=Cursor)
    time_start = time.time()
    db = WkMysql(time_interval=1)
    db.set_table(TABLE)
    for i in range(1000):
        res = db.set_table(TABLE).select_all()
        print(len(res))
        while True:
            for i in range(100000000):
                pass
            break
        db.close()
    time_end = time.time()
    print("time cost:", time_end - time_start)
    # db.execute_many("INSERT INTO test_table(`key`, sno) VALUES(%s, %s)", [[1, "test"], [2, "test2"]])
    # db.execute_many("UPDATE test_table SET `key`=%s WHERE sno=%s", [[10, "test"], [30, "test2"]])
    # db.set_table(TABLE)
    # res = db.execute("UPDATE `test_table` SET `key`=%s WHERE `sno`=%s", [22, "0"])
    # print(res)
    # res = db.update({"id": 1111}, {"id": 1})
    # print(res)
    # print(db.select_all())
    # print(db.select_one(id="31"))
    # print(db.select_one(key="哈哈哈7777"))
    """ exists/exists_by_obj """
    # db.set_table(TABLE)
    # print(db.set_table("gym_reserve").get_column_names())
    # print(db.exists(key="tQ0gK2eM4fR2uD1xK"))
    # print(db.exists(key="tQ0gK2eM4fR2uD1xK1"))
    # print(db.exists(sno=None))
    # print(db.set_table(TABLE).exists(sno=""))
    # print(db.set_table(TABLE).exists({"key": "tQ0gK2eM4fR2uD1xK", "sno": "3123358142", "role": "All"}))
    # print(db.set_table(TABLE).exists({"key": "xE2tX7cN8iZ6xQ1uG", "sno": None}))
    # print(db.set_table(TABLE).exists({"key": "xE2tX7cN8iZ6xQ1uG", "sno": ""}))
    # print(db.set_table(TABLE).exists({"key1": "xE2tX7cN8iZ6xQ1uG", "sno": ""}))
    """ insert_row/insert_rows """
    """ db.set_table(TABLE)
    obj = {"key": "哈哈哈4441", "sno": "2222222222222222222", "role": 1}
    obj2 = {"key": "哈哈哈4444", "sno": "333333333333333333333", "role": ""}
    obj3 = {"key": "哈哈哈55", "sno": "444444444444444444' or 1=1", "role": None}
    id = db.insert_row(obj2)
    print(id)
    obj_list = [obj, obj2, obj3]
    res = db.insert_rows(obj_list)
    print(res)
    res = db.set_table(TABLE).insert_row(id=22, key="wangkang", sno="3123358142", role="All")
    print(res)
    res = db.insert_many(obj_list)
    print(res) """

    """ delete """
    """ obj = {"key": "哈哈哈4441", "sno": "2", "role": 1}
    obj2 = {"key": "哈哈哈4444", "sno": "2", "role": ""}
    obj3 = {"key": "哈哈哈55", "sno": "3", "role": None}
    obj_list = [obj, obj2, obj3]
    print(db.set_table(TABLE).insert_many(obj_list))
    print(db.delete_row(key="哈哈哈4444"))
    print(db.delete_row(sno="2"))
    print(db.delete_row({"key": "3", "sno": 2, "role": None}))
    print(db.delete_row({"key": "3", "sno": 3, "role": 3}))
    print(db.delete_row({"role": "1"}))
    print(db.delete_rows(obj_list))
    for i in range(10):
        print(db.insert_row({"key": i}))
    """
    """ data = []
    for i in range(0, 30):
        data.append({"key": str(i), "sno": str(i), "role": str(i)})
    print(db.insert_rows(data))
    print(db.delete_many(data))

    db.set_table(TABLE)
    print(db.select_all())
    print(db.select(sno=2))
    print(db.select(sno=None))
    print(db.select(role=None))
    print(db.select({"sno": "2", "role": "123"}))
    print(db.select({"sno": "2", "role": 123}))
    print(db.select({"key": "哈哈哈551"}))

    obj = {"sno": "12"}
    obj2 = {"key": "哈哈哈7777", "sno": "112", "role": "1"}
    print(db.update(obj, obj2)) """

    # 测试create_table
    # data = {
    #     "id": "INT PRIMARY KEY AUTO_INCREMENT",
    #     "label": "varchar(255)",
    #     "_id": "varchar(255)",
    #     "creator": "varchar(255)",
    #     "updater": "varchar(255)",
    #     "createTime": "varchar(255)",
    #     "updateTime": "varchar(255)",
    #     "userAgent": "varchar(255)",
    #     "flowDecision": "varchar(255)",
    #     "_widget_1616673287156": "varchar(255)",
    #     "_widget_1679462572706": "varchar(255)",
    #     "_widget_1617678690020": "varchar(255)",
    #     "_widget_1615827614721": "varchar(255)",
    #     "_widget_1615835360787": "varchar(255)",
    #     "_widget_1615835360820": "varchar(255)",
    #     "_widget_1615868277748": "varchar(255)",
    #     "_widget_1679382295430": "varchar(255)",
    #     "_widget_1680158790676": "varchar(255)",
    #     "_widget_1615827613948": "varchar(255)",
    #     "_widget_1615827614024": "varchar(255)",
    #     "_widget_1616127228841": "varchar(255)",
    #     "_widget_1646551883542": "varchar(255)",
    #     "_widget_1615827614179": "varchar(255)",
    #     "_widget_1615828535162": "varchar(255)",
    #     "_widget_1615827614346": "text",
    #     "_widget_1615853253200": "varchar(255)",
    #     "_widget_1646552160980": "varchar(255)",
    #     "_widget_1615827614230": "varchar(255)",
    #     "_widget_1650676573176": "varchar(255)",
    #     "_widget_1616138014817": "varchar(255)",
    #     "_widget_1679319585604": "varchar(255)",
    #     "_widget_1645530113180": "text",
    #     "_widget_1615827614500": "varchar(255)",
    #     "_widget_1615827614519": "varchar(255)",
    #     "_widget_1615827614556": "varchar(255)",
    #     "_widget_1679206290832": "varchar(255)",
    #     "_widget_1679206291318": "varchar(255)",
    #     "_widget_1646573100387": "varchar(255)",
    #     "_widget_1646573100578": "varchar(255)",
    #     "_widget_1616161492340": "varchar(255)",
    #     "_widget_1646573100763": "varchar(255)",
    #     "_widget_1646573103096": "varchar(255)",
    #     "_widget_1615827614467": "varchar(255)",
    #     "_widget_1615868277437": "varchar(255)",
    #     "_widget_1679206291570": "varchar(255)",
    #     "_widget_1679206291997": "varchar(255)",
    #     "_widget_1679206292059": "varchar(255)",
    #     "_widget_1615827614316": "text",
    #     "_widget_1615872450096": "varchar(255)",
    #     "_widget_1615827614331": "text",
    #     "_widget_1615872450115": "varchar(255)",
    #     "_widget_1646573101933": "text",
    #     "_widget_1646573101968": "varchar(255)",
    #     "_widget_1679206292413": "text",
    #     "_widget_1679206292466": "varchar(255)",
    #     "_widget_1646573101063": "varchar(255)",
    #     "_widget_1617845711912": "varchar(255)",
    #     "_widget_1616138013810": "varchar(255)",
    #     "_widget_1710314345323": "varchar(255)",
    #     "_widget_1710314345885": "varchar(255)",
    #     "_widget_1710327754686": "varchar(255)",
    #     "_widget_1710329993986": "varchar(255)",
    #     "chargers_name": "varchar(255)",
    #     "appId": "varchar(255)",
    #     "entryId": "varchar(255)",
    # }

    """ data = {
        "id": "INT PRIMARY KEY AUTO_INCREMENT",
        "key": "varchar(255)",
        "sno": "varchar(255)",
        "role": "varchar(255)",
    }
    db.set_table("test").create_table(data, True)
    db.set_table("test").insert_row({"key": "123", "sno": "456", "role": "789"})
    # db.set_table("test").delete_table() """



from WkLog import INFO
from WkMysql import DB
import time

from WkMysql import log as db_log

# db_log.level = INFO
# db_log.output_location = 1

db = DB(time_interval=10)

from WkLog import log

while True:
    res = db.set_table("test_table").select_all()
    print(len(res))
    time.sleep(1)
    log.debug("test")



from WkMysql import WkMysqlPool
from WkMysql import WkMysql
import time
import threading
from WkLog import log

HOST = "localhost"
PORT = 3306
USER = "root"
PASSWORD = "123456"
DATABASE = "myproject"
TABLE = "test_table"


def multi_thread_test():
    pool = WkMysqlPool(
        host=HOST,
        port=PORT,
        user=USER,
        password=PASSWORD,
        database=DATABASE,
        max_conn=30,
        min_conn=10,
        time_interval=60,
        max_idle_timeout=60 * 60,
    )

    def test_pool(name):
        with pool.get_conn() as conn:
            res = conn.set_table(TABLE).select_all()
            log.info(f"{name} -> {pool.pool.qsize()} -> {len(res)} -> {pool.current_conn}")
            return

    time_start = time.time()
    num = 20
    while True:
        tasks = []
        for i in range(num):
            tasks.append(threading.Thread(target=test_pool, args=("task-{}".format(i),)))
        for task in tasks:
            task.start()
        for task in tasks:
            task.join()
        # time.sleep(1)
        num -= 1
        if num == 0:
            break
        print(num)
    time_end = time.time()
    print("time cost: ", time_end - time_start)


def single_thread_test_pool():
    pool = WkMysqlPool(
        host=HOST,
        port=PORT,
        user=USER,
        password=PASSWORD,
        database=DATABASE,
        max_conn=30,
        min_conn=10,
        time_interval=60,
        max_idle_timeout=60 * 60,
    )

    def test_pool(name):
        with pool.get_conn() as conn:
            res = conn.set_table(TABLE).select_all()
            log.info(f"{name} -> {pool.pool.qsize()} -> {len(res)} -> {pool.current_conn}")
            return

    time_start = time.time()
    num = 20
    while True:
        for i in range(num):
            test_pool(name="task-{}".format(i))
        # time.sleep(1)
        num -= 1
        if num == 0:
            break
        print(num)
    time_end = time.time()
    print("time cost: ", time_end - time_start)


def single_thread_test():
    db = WkMysql(
        host=HOST,
        port=PORT,
        user=USER,
        password=PASSWORD,
        database=DATABASE,
        time_interval=10,
    )

    def test_pool(name):
        res = db.set_table(TABLE).select_all()
        log.info(f"{name} ->  {len(res)}")
        return

    time_start = time.time()
    num = 20
    while True:
        for i in range(num):
            test_pool(name="task-{}".format(i))
        # time.sleep(1)
        num -= 1
        if num == 0:
            break
        print(num)
    time_end = time.time()
    print("time cost: ", time_end - time_start)


if __name__ == "__main__":
    multi_thread_test()
    # single_thread_test()
    # single_thread_test_pool()



# -*- coding: utf-8 -*-
# @Date     : 2024-05-07 10:45:04
# @Author   : WANGKANG
# @Blog     : https://wangkang1717.github.io
# @Email    : 1686617586@qq.com
# @Filepath : WkMysqlPool.py
# @Brief    : WkMysql连接池
# Copyright 2024 WANGKANG, All Rights Reserved.

""" 
项目地址：https://gitee.com/purify_wang/wkdb
"""

from .WkMysql import WkMysql, _log
import time
from queue import Queue
from threading import Condition, Lock, Thread
from contextlib import contextmanager

HOST = "localhost"
PORT = 3306
USER = "root"
PASSWORD = "123456"
DATABASE = "myproject"
TABLE = "test_table"


class WkMysqlPool:
    def __init__(
        self,
        host,
        user,
        password,
        database,
        port,
        min_conn=3,
        max_conn=10,
        max_idle_timeout=60 * 60,  # 最大空闲超时：1小时
        **kwargs,
    ):
        self.host = host
        self.user = user
        self.password = password
        self.database = database
        self.port = port
        self.max_conn: int = max_conn  # 最大连接数
        self.min_conn: int = min_conn  # 最小连接数
        self.max_idle_timeout: int = max_idle_timeout  # 单位：秒
        self.kwargs = kwargs

        self.conditionLock = Condition()
        self.pool: Queue = self._init_pool()

        self.current_conn = self.pool.qsize()  # 当前连接数
        # 启动空闲连接清理线程
        self.new_thread(self.cleanup_idle_threads)

    def _init_pool(self):
        pool = Queue(self.max_conn)
        for _ in range(self.min_conn):
            try:
                conn = self._create_connection()
                pool.put((conn, time.time()))  # 初始化最小连接, 同时记录时间戳
            except Exception as e:
                _log.error(f"Failed to create initial connection: {e}")
                continue
        return pool

    def _create_connection(self) -> WkMysql:
        return WkMysql(
            host=self.host,
            user=self.user,
            password=self.password,
            database=self.database,
            port=self.port,
            **self.kwargs,
        )

    def _get_connection(self) -> WkMysql:
        with self.conditionLock:
            try:
                while self.pool.empty() and self.current_conn >= self.max_conn:
                    self.conditionLock.wait()  # 等待连接池中有空闲连接
                if self.pool.empty():
                    conn = self._create_connection()
                    self.current_conn += 1
                    return conn
                else:
                    conn, _ = self.pool.get()
                    return conn
            except Exception as e:
                _log.error(f"Failed to get connection: {e}")
                return None

    @contextmanager
    def get_conn(self):
        conn = self._get_connection()
        try:
            yield conn  # 提供连接给调用者
        finally:
            # 在上下文退出后释放连接
            self.release_connection(conn)

    def release_connection(self, conn: WkMysql):
        with self.conditionLock:
            try:
                self.pool.put((conn, time.time()))
            except Exception as e:
                _log.error(f"Failed to release connection: {e}")
            finally:
                self.conditionLock.notify()  # 通知其他线程有空闲连接

    def close_connections(self, conn):
        with self.conditionLock:
            try:
                conn.close()  # 关闭空闲连接
            except Exception as e:
                _log.error(f"Failed to close connection: {e}")
            finally:
                self.current_conn -= 1
                self.conditionLock.notify()

    def new_thread(self, func, *args):
        t = Thread(target=func, args=args)
        t.daemon = True
        t.start()

    def cleanup_idle_threads(self):
        while True:
            time.sleep(self.max_idle_timeout)
            _log.debug("cleanup_idle_threads")
            temp_pool = []
            while not self.pool.empty():
                conn, last_use_time = self.pool.get()
                if time.time() - last_use_time > self.max_idle_timeout and self.current_conn > self.min_conn:
                    self.close_connections(conn)
                    _log.debug(f"Closed idle connection: {conn}")
                else:
                    temp_pool.append((conn, last_use_time))
            for conn, last_use_time in temp_pool:
                self.release_connection(conn)



# -*- coding: utf-8 -*-
# @Date     : 2023-10-13 13:00:00
# @Author   : WangKang
# @Blog     :
# @Email    : 1686617586@qq.com
# @Filepath : WkMysql.py
# @Brief    : 封装数据库操作，适合单线程使用，且短时间连接数据库
# Copyright 2023 WANGKANG, All Rights Reserved.

""" 
项目地址：https://gitee.com/purify_wang/wkdb
"""

import pymysql
from pymysql import cursors
import sys
from threading import Thread, Lock
import time
import atexit
from WkLog import WkLog

HOST = "localhost"
PORT = 3306
USER = "root"
PASSWORD = "123456"
DATABASE = "myproject"
TABLE = "test_table"

_log = WkLog()


class WkMysql:
    def __init__(
        self,
        host=HOST,
        user=USER,
        password=PASSWORD,
        database=DATABASE,
        port=PORT,
        cursorclass=cursors.DictCursor,
        time_interval=60,  # 设置每隔多长时间进行一次连接测试，单位秒，目的是保持连接不断开
        **kwargs,
    ):
        self.host = host
        self.user = user
        self.password = password
        self.database = database
        self.port = port
        self.cursorclass = cursorclass
        self.time_interval = time_interval
        self.kwargs = kwargs

        self.table = None
        self.conn = self.connect_db()
        self.close_flag = False

        _log.debug(f"Keep connect to database every {time_interval} seconds!")
        self.new_thread(self.keep_connect, self.time_interval)
        atexit.register(self.close)
        self.lock = Lock()

    def connect_db(self) -> pymysql.Connection:
        try:
            conn = pymysql.connect(
                host=self.host,
                user=self.user,
                port=self.port,
                passwd=self.password,
                database=self.database,
                autocommit=True,
                cursorclass=self.cursorclass,
                **self.kwargs,
            )
            _log.debug("Successfully connected to database!")
            return conn
        except Exception as e:
            msg = f"Failed to connect to database! -> {str(e)}"
            _log.error(msg)
            raise Exception(msg)

    def close(self):
        _log.debug("Close database connection!")
        with self.lock:
            try:
                if self.close_flag:
                    return
                self.conn.close()
                self.close_flag = True
            except Exception as e:
                _log.error(f"Failed to close database connection! -> {str(e)}")

    def before_execute(func):
        def wrapper(self, *args, **kwargs):
            if self.table is None:
                raise Exception("table is not set!")
            with self.lock:
                self.__test_conn()
                result = func(self, *args, **kwargs)
            return result

        return wrapper

    def new_thread(self, func, *args):
        # print(args)
        t = Thread(target=func, args=args)
        t.daemon = True
        t.start()

    def keep_connect(self, time_interval):
        while True:
            time.sleep(time_interval)
            with self.lock:
                _log.debug("Keep connect to database!")
                self.__test_conn()

    def __test_conn(self):
        """
        长连接时，如果长时间不进行数据库交互，连接就会关闭，再次请求就会报错
        每次使用游标的时候，都调用下这个方法
        """
        # _log.debug("__test_conn")
        # self.conn.ping()
        try:
            if self.close_flag:
                return
            self.conn.ping()
        except:
            self.conn = self.connect_db()

    def __get_query_params(self, obj: dict | list):
        if isinstance(obj, dict):
            return " AND ".join([f"`{column_name}` {'=' if obj[column_name] is not None else 'is'} %s" for column_name in obj.keys()])
        elif isinstance(obj, list):
            return " AND ".join([f"`{column_name}` {'=' if column_name is not None else 'is'} %s" for column_name in obj])

    def __get_set_params(self, obj: dict):
        return ", ".join([f"`{column_name}` {'=' if obj[column_name] is not None else 'is'} %s" for column_name in obj.keys()])

    def __get_col_params(self, obj: dict | list):
        if isinstance(obj, dict):
            return ", ".join([f"`{column_name}`" for column_name in obj.keys()])
        elif isinstance(obj, list):
            return ", ".join([f"`{column_name}`" for column_name in obj])

    def __get_values(self, obj: dict | list):
        if isinstance(obj, dict):
            return list(obj.values())
        elif isinstance(obj, list):
            res = []
            for o in obj:
                res.append(self.__get_values(o))
            return res

    def __get_placeholders(self, length):
        return ", ".join(["%s"] * length)

    def __validate_args(self, args, kwargs):
        """
        验证参数是否正确
        """
        if not args and not kwargs:
            raise Exception("args or kwargs must be used!")
        if args and kwargs:
            raise Exception("args and kwargs cannot be used together!")
        if args:
            if len(args) > 1 or not isinstance(args[0], dict):
                raise Exception("args's length must be 1 and the type must be dict!")

    def __print_info(self, cursor, func_name, success=True, error_msg=None):
        if success:
            _log.debug(f"Success: {func_name} -> {cursor._executed if type(cursor._executed) == str else cursor._executed.decode()} -> Rows affected: {cursor.rowcount}")
        else:
            _log.error(f"Failure: {func_name} -> {f'{cursor._executed if type(cursor._executed) == str else cursor._executed.decode()}' if cursor._executed else 'None'} -> {error_msg}")

    def set_table(self, table):
        self.table = table
        return self

    @before_execute
    def create_table(self, obj: dict, delete_if_exists=False):
        """
        创建表
        :param obj: 字典对象，键为列名，值为列类型
        :param delete_if_exists: 是否删除原有表
        :return: True/False
        """
        col_params = ", ".join([f"`{column_name}` {column_type}" for column_name, column_type in obj.items()])
        if delete_if_exists:
            self.delete_table()

        sql = f"CREATE TABLE IF NOT EXISTS {self.table} ({col_params})"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql)
                self.__print_info(cursor, sys._getframe().f_code.co_name)
            return True
        except pymysql.Error as e:
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return False

    def delete_table(self):
        """
        删除表
        :return: True/False
        """
        sql = f"DROP TABLE IF EXISTS {self.table}"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql)
                self.__print_info(cursor, sys._getframe().f_code.co_name)
            return True
        except pymysql.Error as e:
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return False

    def get_column_names(self):
        sql = "SELECT COLUMN_NAME FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql, (self.database, self.table))
                res = []
                for row in cursor.fetchall():
                    res.append(row.get("COLUMN_NAME"))
                self.__print_info(cursor, sys._getframe().f_code.co_name)
            return res
        except pymysql.Error as e:
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return []

    @before_execute
    def exists(self, *args, **kwargs):
        """
        根据字典对象判断元素是否存在
        - demo:
            - exists({"id": 1, "name": "wangkang"})
            - exists({id=1, name=wangkang})
        """
        self.__validate_args(args, kwargs)
        obj = args[0] if args else kwargs

        values = self.__get_values(obj)
        params = self.__get_query_params(obj)
        sql = f"SELECT 1 FROM {self.table} WHERE {params} LIMIT 1"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql, values)
                flag = cursor.fetchone() != None
                self.__print_info(cursor, sys._getframe().f_code.co_name)
            return flag
        except pymysql.Error as e:
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return False

    @before_execute
    def insert_row(self, *args, **kwargs):
        """
        插入一行数据
        :return: True/False, insert_id
        - demo:
            - insert_row({"id": 1, "name": "wangkang"})
            - insert_row(id=1, name=wangkang)
        """
        self.__validate_args(args, kwargs)
        obj = args[0] if args else kwargs

        values = self.__get_values(obj)
        col_params = self.__get_col_params(obj)
        placeholders = self.__get_placeholders(len(obj))
        sql = f"INSERT INTO {self.table}({col_params}) VALUES({placeholders})"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql, values)
                self.conn.commit()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
                return cursor.rowcount, cursor.lastrowid  # 获取插入数据的自增ID,如果没有自增ID，则返回0
        except pymysql.Error as e:
            self.conn.rollback()
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return -1, -1

    def insert_rows(self, obj_list: list[dict]):
        """
        插入多行数据，不会因为个别数据的添加失败导致后面的所有数据都插入失败
        :param obj_list: 列表，元素为字典对象，键为列名，值为列值
        :return: 字典对象，键为success和fail，值为成功和失败次数
        """
        if not obj_list:
            return None
        success = 0
        fail = 0
        for obj in obj_list:
            flag, _ = self.insert_row(obj)
            if flag > 0:
                success += 1
            else:
                fail += 1
        return {"success": success, "fail": fail}

    @before_execute
    def insert_many(self, obj_list: list[dict]):
        """
        使用executemany来批量插入数据 此操作具有原子性
        obj_list: 列表，元素为字典对象，键为列名，值为列值
        :return: True/False
        """
        if not obj_list:
            _log.warn("要插入的数据为空!")
            return
        values = self.__get_values(obj_list)
        col_params = self.__get_col_params(obj_list[0])
        placeholders = self.__get_placeholders(len(obj_list[0].keys()))
        sql = f"INSERT INTO {self.table}({col_params}) VALUES({placeholders})"
        try:
            with self.conn.cursor() as cursor:
                cursor.executemany(sql, values)
                self.conn.commit()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
                return cursor.rowcount
        except Exception as e:
            self.conn.rollback()
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return False

    @before_execute
    def delete_row(self, *args, **kwargs):
        """
        根据条件删除一行数据
        - demo:
            - delete_row({"id": 1, "name": "wangkang"})
            - delete_row(id=1, name=wangkang)
        """
        self.__validate_args(args, kwargs)
        obj = args[0] if args else kwargs

        values = self.__get_values(obj)
        params = self.__get_query_params(obj)
        sql = f"DELETE FROM {self.table} WHERE {params}"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql, values)
                self.conn.commit()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
                return cursor.rowcount
        except pymysql.Error as e:
            self.conn.rollback()
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return -1

    def delete_rows(self, obj_list: list):
        """
        :param obj_list: 列表，元素为字典对象，键为列名，值为列值
        :return: 字典对象，键为success和fail，值为成功和失败次数
        """
        if not obj_list:
            return None
        success = 0
        fail = 0
        for obj in obj_list:
            if self.delete_row(obj) > 0:
                success += 1
            else:
                fail += 1
        return {"success": success, "fail": fail}

    @before_execute
    def delete_many(self, obj_list: list[dict]):
        """
        使用executemany来批量插入数据 此操作具有原子性
        :param obj_list: 列表，元素为字典对象，键为列名，值为列值
        :return: True/False
        """
        if not obj_list:
            _log.warn("要删除的数据为空!")
            return
        values = self.__get_values(obj_list)
        params = self.__get_query_params(obj_list[0])
        # print(params)
        sql = f"DELETE FROM {self.table} WHERE {params}"
        try:
            with self.conn.cursor() as cursor:
                cursor.executemany(sql, values)
                self.conn.commit()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
                return cursor.rowcount
        except pymysql.Error as e:
            self.conn.rollback()
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return -1

    @before_execute
    def select_all(self):
        """
        查询所有数据
        :return: 列表，元素为字典对象，键为列名，值为列值
        """
        sql = f"SELECT * FROM {self.table}"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql)
                data = cursor.fetchall()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
            return data
        except pymysql.Error as e:
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return None

    @before_execute
    def select(self, *args, **kwargs):
        """
        根据条件进行查询
        :return: 列表，元素为字典对象，键为列名，值为列值
        - demo:
            - select({"id": 1, "name": "wangkang"})
            - select(id=1, name=wangkang)
        """
        self.__validate_args(args, kwargs)
        obj = args[0] if args else kwargs

        values = self.__get_values(obj)
        param = self.__get_query_params(obj)
        # print(f"|{col_names}|{col_params}|{values}|{param}|")
        sql = f"SELECT * FROM {self.table} where {param}"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql, values)
                data = cursor.fetchall()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
            return data
        except pymysql.Error as e:
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return None

    @before_execute
    def select_one(self, *args, **kwargs):
        """
        根据条件进行查询，只返回第一条数据
        :return: 字典对象，键为列名，值为列值
        - demo:
            - select_one({"id": 1, "name": "wangkang"})
            - select_one(id=1, name=wangkang)
        """
        self.__validate_args(args, kwargs)
        obj = args[0] if args else kwargs

        values = self.__get_values(obj)
        param = self.__get_query_params(obj)
        sql = f"SELECT * FROM {self.table} where {param} LIMIT 1"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql, values)
                data = cursor.fetchone()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
            return data
        except pymysql.Error as e:
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return None

    @before_execute
    def update(self, target_obj: dict, new_obj: dict):
        """
        根据条件更新数据
        :param target_obj: 字典对象，键为列名，值为列值 作为更新条件
        :param new_obj: 字典对象，键为列名，值为列值 作为更新内容
        :return: True/False
        """
        values = self.__get_values(new_obj) + self.__get_values(target_obj)
        set_params = self.__get_set_params(new_obj)
        query_params = self.__get_query_params(target_obj)
        sql = f"UPDATE {self.table} set {set_params} where {query_params}"
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql, values)
                self.conn.commit()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
                return cursor.rowcount
        except pymysql.Error as e:
            self.conn.rollback()
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return -1

    @before_execute
    def execute(self, sql, values=None):
        """
        执行SQL语句
        :param sql: SQL语句
        :param values: 列表，元素为SQL语句中占位符对应的值
        :return: 影响行数

        - demo:
            - execute("INSERT INTO table_name(col1, col2) VALUES(%s, %s)", [1, "test"])
            - execute("UPDATE table_name SET col1=%s WHERE col2=%s", [2, "test"])
        """
        try:
            with self.conn.cursor() as cursor:
                cursor.execute(sql, values)
                self.conn.commit()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
                return cursor.rowcount
        except pymysql.Error as e:
            self.conn.rollback()
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return -1

    def execute_many(self, sql, values_list):
        """
        批量执行SQL语句
        :param sql: SQL语句
        :param values_list: 列表，元素为列表，每个子列表为SQL语句中占位符对应的值
        :return: 影响行数

        - demo:
            - execute_many("INSERT INTO table_name(col1, col2) VALUES(%s, %s)", [[1, "test"], [2, "test2"]])
            - execute_many("UPDATE table_name SET col1=%s WHERE col2=%s", [[2, "test"], [3, "test2"]])
        """
        try:
            with self.conn.cursor() as cursor:
                cursor.executemany(sql, values_list)
                self.conn.commit()
                self.__print_info(cursor, sys._getframe().f_code.co_name)
                return cursor.rowcount
        except pymysql.Error as e:
            self.conn.rollback()
            self.__print_info(cursor, sys._getframe().f_code.co_name, success=False, error_msg=str(e))
            return -1



import threading

currentRunThreadNumber = 0
maxSubThreadNumber = 10


def task():
    global currentRunThreadNumber
    thName = threading.currentThread().name

    condLock.acquire()  # 上锁
    print("start and wait run thread : %s" % thName)

    condLock.wait()  # 暂停线程运行、等待唤醒
    currentRunThreadNumber += 1
    print("carry on run thread : %s" % thName)

    condLock.release()  # 解锁


if __name__ == "__main__":
    condLock = threading.Condition()

    for i in range(maxSubThreadNumber):
        subThreadIns = threading.Thread(target=task)
        subThreadIns.start()

    while currentRunThreadNumber < maxSubThreadNumber:
        notifyNumber = int(
            input("Please enter the number of threads that need to be notified to run：")
        )

        condLock.acquire()
        condLock.notify(notifyNumber)  # 放行
        condLock.release()

    print("main thread run end")

# 先启动10个子线程，然后这些子线程会全部变为等待状态
# start and wait run thread : Thread-1
# start and wait run thread : Thread-2
# start and wait run thread : Thread-3
# start and wait run thread : Thread-4
# start and wait run thread : Thread-5
# start and wait run thread : Thread-6
# start and wait run thread : Thread-7
# start and wait run thread : Thread-8
# start and wait run thread : Thread-9
# start and wait run thread : Thread-10

# 批量发送通知，放行特定数量的子线程继续运行
# Please enter the number of threads that need to be notified to run：5  # 放行5个
# carry on run thread : Thread-4
# carry on run thread : Thread-3
# carry on run thread : Thread-1
# carry on run thread : Thread-2
# carry on run thread : Thread-5

# Please enter the number of threads that need to be notified to run：5  # 放行5个
# carry on run thread : Thread-8
# carry on run thread : Thread-10
# carry on run thread : Thread-6
# carry on run thread : Thread-9
# carry on run thread : Thread-7

# Please enter the number of threads that need to be notified to run：1
# main thread run end



# -*- coding: utf-8 -*-
# @Date     : 2024-07-31 10:45:04
# @Author   : WANGKANG
# @Blog     : https://wangkang1717.github.io
# @Email    : 1686617586@qq.com
# @Filepath : setup.py
# @Brief    : 打包配置文件
# Copyright 2024 WANGKANG, All Rights Reserved.

# 更新软件包：pip install --upgrade setuptools wheel
# 安装twine: pipenv install twine --dev
# 运行命令: python setup.py upload

import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = "WkMysql"
DESCRIPTION = "The secondary encapsulation of pymysql provides a more user-friendly interface and thread pool, and will support SQLite in the future."
URL = "https://github.com/WANGKANG1717/WkDB"  # Github
# URL = "https://gitee.com/purify_wang/git_name" # Gitee
AUTHOR = "WANGKANG"
EMAIL = "1686617586@qq.com"
REQUIRES_PYTHON = ">=3.10.0"
LICENSE = "GPL-2.0"
PLATFORMS = ["all"]
INSTALL_REQUIRES = ["pymysql", "WkLog"]
EXTRAS_REQUIRE = {
    # 'fancy feature': ['django'],
}

VERSION = ""  # 为空自动加载包内__init__.py文件里的__version__变量
LONG_DESCRIPTION = ""  # 为空自动加载readme.md文件

here = os.path.abspath(os.path.dirname(__file__))


def load_version():
    """加载版本号"""
    global VERSION
    if not VERSION:
        project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
        with open(os.path.join(here, project_slug, "__init__.py")) as f:
            for line in f.readlines():
                if line.strip().startswith("__version__"):
                    VERSION = line.split("=")[1].strip().strip("'").strip('"')
                    break

    if not VERSION:
        raise RuntimeError("Cannot find version information")


def load_readme():
    """自动加载readme.md文件，如果没有则使用描述信息"""
    global LONG_DESCRIPTION
    try:
        with open(os.path.join(here, "readme.md"), encoding="utf-8") as f:
            LONG_DESCRIPTION = f.read()
    except FileNotFoundError:
        LONG_DESCRIPTION = DESCRIPTION


def print_info():
    print("=======================================================")
    print("Current Python Platform: ", sys.platform)
    print("Current Python Prefix: ", sys.prefix)
    print("Current Python Version: ", sys.version_info)
    print("Current Python Path: ", sys.executable)
    print("=======================================================")
    print("NAME: ", NAME)
    print("DESCRIPTION: ", DESCRIPTION)
    print("URL: ", URL)
    print("AUTHOR: ", AUTHOR)
    print("EMAIL: ", EMAIL)
    print("REQUIRES_PYTHON: ", REQUIRES_PYTHON)
    print("VERSION: ", VERSION)
    print("LICENSE: ", LICENSE)
    print("PLATFORMS: ", PLATFORMS)
    print("INSTALL_REQUIRES: ", INSTALL_REQUIRES)
    print("EXTRAS_REQUIRE: ", EXTRAS_REQUIRE)
    print("LONG_DESCRIPTION: ", LONG_DESCRIPTION)
    print("=======================================================")


load_version()
load_readme()


class UploadCommand(Command):
    """构建、并上传发布命令、推动标签到远程仓库"""

    user_options = []

    @staticmethod
    def status(s):
        """加粗输出文字"""
        print(f"\033[1m{s}\033[0m")

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        print_info()
        confirm = input("是否继续? yes/no:   ")
        if confirm.lower() != "yes":
            print("取消安装!")
            sys.exit(0)

        try:
            self.status("1. 移除之前的构建文件…")
            rmtree(os.path.join(here, "dist"))
            rmtree(os.path.join(here, "build"))
            rmtree(os.path.join(here, "*.egg-info"))
        except OSError:
            pass

        self.status("2. 构建Source and Wheel (universal)文件…")
        flag = os.system(f"{sys.executable} setup.py sdist bdist_wheel --universal")
        if flag != 0:
            raise RuntimeError("构建失败!")

        self.status("3. 通过Twine上传到PyPI…")
        flag = os.system("twine upload dist/*")
        if flag != 0:
            raise RuntimeError("上传失败!")

        self.status("4. 推送标签到远程仓库…")
        flag = os.system(f"git tag v{VERSION}")
        if flag != 0:
            raise RuntimeError("创建标签失败!")
        flag = os.system("git push --tags")
        if flag != 0:
            raise RuntimeError("推送标签失败!")

        sys.exit()


setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    python_requires=REQUIRES_PYTHON,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],
    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    include_package_data=True,
    license=LICENSE,
    platforms=PLATFORMS,
    classifiers=[
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Natural Language :: Chinese (Simplified)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Software Development :: Libraries",
    ],
    cmdclass={
        "upload": UploadCommand,
    },
)
